/* Minimal Impact SHA-256 */

#ifdef HAVE_OPENSSL_FIPS_SHA_H
#include <openssl/fips_sha.h>
#endif

#ifdef HAVE_OPENSSL_SHA_H
#include <openssl/sha.h>
#endif

#ifdef HAVE_SHA256_H
#include <sha256.h>
#endif

/* Check for mac-specific SHA-256, which has slightly different names. */
#if !defined(HAVE_SHA256_INIT) && defined(HAVE_CC_SHA256_INIT) 
#include <CommonCrypto/CommonDigest.h>
#define SHA256_Init   CC_SHA256_Init
#define SHA256_Update CC_SHA256_Update
#define SHA256_Final  CC_SHA256_Final
#define SHA256_CTX    CC_SHA256_CTX
#define HAVE_SHA256_INIT
#endif

/* Bring in SHA256.c if it is not built in */
#if !defined(HAVE_SHA256_INIT)
#include "sha256.c"
#define HAVE_SHA256_INIT
#endif

#if defined(HAVE_SHA256_INIT) && !defined(HAVE_SHA256)
extern "C" {
    static int SHA256(const unsigned char *d,size_t s,unsigned char *md)
    {
	SHA256_CTX cc;
	SHA256_Init(&cc);
	SHA256_Update(&cc,d,s);
	SHA256_Final(md,&cc);
	return 0;
    }
}
#define HAVE_SHA256
#endif

#ifndef SHA256_DIGEST_LENGTH
#define SHA256_DIGEST_LENGTH 32
#endif

#if !defined(HAVE_SHA256)
#error SHA256 is required for proper operation
#endif

