/*
 * utils.h:
 * Some useful utilities for building AFF-aware programs.
 */

#ifndef UTILS_H
#define UTILS_H

#include <algorithm>
#include <cstdlib>
#include <vector>
#include <string>
#include <map>
#include <iostream>
#ifdef HAVE_OPENSSL_PEM_H
#include <openssl/x509.h>
#include <openssl/pem.h>
#else
typedef void X509;
typedef void EVP_PKEY;
typedef void BIO;
#define BIO_free free
#endif

using namespace std;

bool ends_with(string str,string ending);


/* Structure for hash map */
struct less_c_str
{
    inline bool operator()( const char* x, const char* y) const
    {     return ( strcmp( x,y ) < 0 );
    }
};

struct md5blob {
    unsigned char buf[16];
};

typedef map< const char*, struct md5blob, less_c_str > hashMapT;

/* The seginfo stores information about a segment other than its data*/
class seginfo {
public:
    seginfo(string n1,size_t l1,u_int a1): name(n1),len(l1),arg(a1) {}
    string name;
    size_t len;
    u_long arg;
    int64_t pagenumber() const {return af_segname_page_number(name.c_str());}
    bool inline operator==(const class seginfo &b) const {
	return name == b.name;
    }
};

class seglist : public vector<seginfo> {
public:
    bool contains(string name){ 
	for(vector<seginfo>::const_iterator i = begin(); i!=end(); i++){
	    if(i->name == name) return true;
	}
	return false;
    }
    bool has_signed_segments(){		// returns true if any segments are signed
	for(seglist::const_iterator seg = begin(); seg!=end() ;seg++){
	    if(ends_with(seg->name,AF_SIG256_SUFFIX)){
		return true;
	    }
	}
	return false;
    }
    int get_seglist(AFFILE *af);
    seglist(){}
    seglist(AFFILE *af){
	get_seglist(af);
    }
};


class outelement {
public:
    outelement(){}
    AFFILE *af;			// where output goes
    seglist segs;			// list of existing segments in output
};
typedef vector<outelement> outlist;

int  parse_chain(const string &name);
int  highest_chain(seglist &slist);

#ifdef USE_AFFSIGS
class aff_bom {
    X509 *cert;
    EVP_PKEY *privkey;
    char *notes;
public:
    bool opt_note;
    BIO *xml;
    aff_bom(bool flag):xml(0),cert(0),privkey(0),notes(0),opt_note(flag) { }
    ~aff_bom(){
	if(notes) free(notes);
	if(xml) BIO_free(xml);
    }
    int read_files(const char *cert_file,const char *key_file);	// returns 0 if success
    void add(const char *segname,int sigmode,const u_char *seghash,size_t seghash_len);
    void close();			// close the BoM
    int  write(AFFILE *af,seglist &segments);	// write the BoM
    char *get_notes();
};
#endif


#endif
